Package: multiplex
Type: Package
Title: Algebraic Tools for the Analysis of Multiple Social Networks
Version: 4.0
Depends: R (>= 4.6.0)
Imports: methods
Suggests: multigraph, Rgraphviz, knitr
Description: Algebraic procedures for analyses of multiple social networks are provided with this 
    package as described in Ostoic (2020) <DOI:10.18637/jss.v092.i11>. 'multiplex' supports the 
    creation and analysis of multiplex, multimode, and multilevel network data in various formats. 
    It combines methods based on partially ordered semigroups, decomposition procedures, semiring 
    structures, and relational bundles for multivariate network analysis. It also provides an 
    algebraic approach to affiliation networks using Galois derivations, with visualization options.
Date: 2026-07-29
Authors@R: person(given = "Antonio", family = "Rivero Ostoic", role = c("aut", "cre"), email = "multiplex@post.com")
Author: Antonio Rivero Ostoic [aut, cre]
Maintainer: Antonio Rivero Ostoic <multiplex@post.com>
URL: https://github.com/mplex/multiplex/
BugReports: https://github.com/mplex/multiplex/issues/
Repository: CRAN
License: GPL-3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-07-29 15:38:41 UTC; AOST
Date/Publication: 2026-07-30 09:30:02 UTC
Built: R 4.6.1; ; 2026-07-30 13:25:10 UTC; unix
